<?php
/*
Plugin Name: Tweet Blender
Plugin URI: http://kirill-novitchenko.com/tweet-blender/
Description: Provides sidebar widget and archive page that shows tweets for multiple authors or hashtags blended together.
Version: 1.0.1
Author: Kirill Novitchenko
Author URI: http://kirill-novitchenko.com
*/

/*  Copyright 2009  Kirill Novitchenko  (email : knovitchenko@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// include JSON library
class_exists('Services_JSON') || require('JSON.php');

// register admin page
add_action('admin_menu', 'tweet_blender_admin_menu');
function tweet_blender_admin_menu() {
  add_options_page('Tweet Blender Settings', 'Tweet Blender', 8, __FILE__, 'tweet_blender_admin_page');
}

// register stylesheet
add_action('wp_head', 'add_header_code');
function add_header_code() {

	echo '<link type="text/css" rel="stylesheet" href="' . get_bloginfo('wpurl') . '/wp-content/plugins/tweet-blender/css/tweets.css" />' . "\n";
}

// hookup widget action for sidebar
add_action('get_sidebar', 'tb_widget');

// hookup filter to add tweet list to the content of archive page
add_filter('the_content', 'tb_add_archive_page_content');
function tb_add_archive_page_content($content = '') {
	global $post;
	
	// work with pages only
	if ($post->post_type != 'page') {
		return $content;
	}
	
	// if looking at archive page, apend list of tweets to content
	if ($post->ID == tb_get_archive_post_id()) {
		return $content . '<div id="tweetblenderarchive">' . "\n" . tb_make_tweet_list('archive') . "\n" . '</div>';
	}
	// else, do nothing
	else {
		return $content;
	}
}


/**
 * Create HTML for the sidebar widget
 *
 * @uses tb_get_archive_post_id()
 * @uses tb_make_tweet_list()
 */
function tb_widget() {

	// create HTML for widget
	$widget_html = '<div id="tweetblender">';
	
	$widget_html .= '<div id="twitterlogo"></div>';
	$widget_html .= '<a id="infolink" href="http://kirill-novitchenko.com/tweet-blender/" target="_blank" title="TweetBlender info"> </a>';

	$widget_html .= tb_make_tweet_list('widget');
	
	if ($archive_post_id = tb_get_archive_post_id()) {
		$widget_html .= '<a href="' . get_permalink($archive_post_id) . '">view all &raquo;</a>';
	}
	$widget_html .= '</div>';
	
	echo $widget_html;
}


/**
 * Create HTML for the list of tweets
 *
 * @uses tb_get_blended_tweets()
 * @uses tb_verbal_time()
 * @uses get_option()
 * 
 * @param string $mode The mode of list building - widget or archive.
 */
function tb_make_tweet_list($mode = 'widget') {

	// see how many tweets to show
    $max = get_option('tb_' . $mode . '_tweets_num');
	
	// see if we need to show photos
    $tb_show_photos = get_option('tb_' . $mode . '_show_photos');

	// see if we need to show source
    $tb_show_source = get_option('tb_' . $mode . '_show_source');
	
	// check what date format to use
	$tb_timestamp_format = get_option('tb_general_timestamp_format');
	
	// check if we need to link urls, screen names or hashtags
	$tb_link_urls = get_option('tb_general_link_urls');
	$tb_link_screen_names = get_option('tb_general_link_screen_names');
	$tb_link_hashtags = get_option('tb_general_link_hash_tags');
	
	// get all tweets from all sources
	$tweets = tb_get_blended_tweets();
	
	// if there was an error, return it as HTML comment
	if (array_key_exists('error', $tweets)) {
		return '<!-- ' . $tweets['error'] . ' --><br/>Feeds temporary unavailable...<br/>';
	}
	
	// if there was just one minor error, put it as comment before the list
	if (array_key_exists('notice', $tweets)) {
		$list_html .= '<!-- ' . $tweets['notice'] . ' -->';
		unset($tweets['0']);
	}

	// build html for the list
	$list_html .= '<ul>';
	$shown_tweets = 0;
	foreach ($tweets as $timestamp => $tweet) {
		if ($max > 0 && $shown_tweets == $max) {
			break;
		}
		
		// add screen name if from_user is given
		if (!$tweet->{'user'}->{'screen_name'} && $tweet->{'from_user'}) {
			$tweet->{'user'}->{'screen_name'} = $tweet->{'from_user'};
		}
		
		$list_html .= '<li class="tb_tweet">';
		
		// show photo if requested
		if ($tb_show_photos) {

			// add image url
			if (!$tweet->{'user'}->{'profile_image_url'} && $tweet->{'profile_image_url'}) {
				$tweet->{'user'}->{'profile_image_url'} = $tweet->{'profile_image_url'};
			}

			$list_html .= '<a class="tb_photo" href="http://twitter.com/' . $tweet->{'user'}->{'screen_name'} . '" target="_blank">';
			$list_html .= '<img src="' . $tweet->{'user'}->{'profile_image_url'} . '" alt="' . $tweet->{'user'}->{'name'} . '"/>';
			$list_html .= '</a>';
		}

		// show author
		$list_html .= '<span class="tb_author"><a href="http://twitter.com/' . $tweet->{'user'}->{'screen_name'} . '" target="_blank">' . $tweet->{'user'}->{'screen_name'} . '</a>: </span> ';

		// show text
		$text = $tweet->{'text'};
		if ($tb_link_urls) {
			$text = preg_replace('/(https?:\/\/\S+)/','<a href="$1" target="_blank">$1</a>',$text);
		}
		if ($tb_link_screen_names) {
			$text = preg_replace('/\@([^\s\.\:]+)/','RT @<a href="http://twitter.com/$1" target="_blank">$1</a>',$text);
		}
		if ($tb_link_hashtags) {
			$text = preg_replace('/#([^\s\,]+)/','#<a href="http://search.twitter.com/search?q=%23$1" target="_blank">$1</a>',$text);
		}
		$list_html .= '<san class="tb_mgs">' . $text . '</span><br/>';

		// start tweet footer with info
		$list_html .= ' <span class="tb_tweetinfo">';
		
		// show timestamp
		$list_html .= '<a href="http://twitter.com/' . $tweet->{'user'}->{'screen_name'} . '/statuses/' . $tweet->{'id'} . '">';
		if ($tb_timestamp_format) {
			$list_html .= date($tb_timestamp_format,$timestamp);
		}
		else {
			$list_html .= tb_verbal_time($timestamp);
		} 
		$list_html .= '</a>';
		
		// show source if requested
		if ($tb_show_source && $tweet->{'source'}) {
			$list_html .= ' from ' . html_entity_decode($tweet->{'source'});
		}
		
		// end tweet footer
		$list_html .= '</span>';
			
		// end tweet	
		$list_html .= "</li>\n";
		
		$shown_tweets++;
	}

	$list_html .= '<li class="tb_tweet last"></li>';
	$list_html .= '</ul>';
	
	// return html
	return $list_html;	
}

function tb_get_archive_post_id() {
	// if we already have page id saved as option, return it
	if ($post_id = get_option('tb_archive_page_id')) {
		return $post_id;
	}
	// else if we have such a page already, get its id and store as option
	else if ($post = get_page_by_path('tweets-archive')) {
		$post_id = $post->post_name;
		update_option('tb_archive_page_id',$post_id);
		return $post_id;
	}
	// else create such a page 
	else if ($post_id = wp_insert_post(array(
			  'post_status' => 'publish',
			  'post_type' => 'page',
			  'post_author' => 1,
			  'post_title' => 'Twitter Feed',
			  'post_content' => 'Our twitter feed.',
			  'post_name' => 'tweets-archive'
		))) {
			update_option('tb_archive_page_id',$post_id);
			return $post_id;
	}
	else {
		return null;
	}
}



function tb_get_blended_tweets() {

    // Read in existing option values from database
    $tb_source_ids = get_option('tb_general_source_ids');

	$sources = split(',',$tb_source_ids);
	$all_tweets = array();
	$errors = array();
	
	// get tweets from each source
	foreach ($sources as $sourceId) {

		$tweets = tb_get_tweets($sourceId);
		// if there were errors, add them to list
		if(array_key_exists('error', $tweets)) {
			$errors[] = $tweets['error'];
		}
		else {
			foreach ($tweets as $timestamp => $data){
				$all_tweets[$timestamp] = $data;
			}
		}
	}
	
	// if we have at least some tweets
	if (sizeof($all_tweets) > 0) {

		// sort all tweets by key (timestamp)
		krsort($all_tweets);
		
		// if there were some errors, push them as a zero-timestamped item
		if (sizeof($errors) > 0) {
			$all_tweets['notice'] = join("\n",$errors);
		}
		
		return $all_tweets;
	}
	else {
		return array('error' => join("\n",$errors));
	}
}

function tb_get_tweets($sourceId) {
	
	// get feed
	$data_set = tb_get_twitter_feed($sourceId);
	
	// if we got string back, it's an error message
	if (is_string($data_set)) {
		return array('error' => $sourceId . ' feed: ' . $data_set);
	}
	// else, process feed and get tweets from it
	else {
		$source_tweets = array();
		if (stripos($sourceId,'#') === 0) {
			$tweets = $data_set->{results};
		}
		else {
			$tweets = $data_set;
		}
		foreach ($tweets as $tweet) {
			$source_tweets[tb_strtotime($tweet->{'created_at'})] = $tweet;
		}

		return $source_tweets;
	}
}



/**
 * Get data in JSON format for a given source Id (screen name or hashtag)
 * Uses cache if possible
 *
 * @uses tb_rate_limit_remaining()
 */
function tb_get_twitter_feed($sourceId) {
	
	// get TTL for cache
    $tb_cache_ttl_seconds = get_option('tb_general_cache_ttl');
	
	// default to 1 hour TTL to be on the safe side
	if (!$tb_cache_ttl_seconds) {
		$tb_cache_ttl_seconds = 60 * 60;
	}
	
	// form cache file name
	$tb_cache_file = dirname(__FILE__) . '/cache/' . $sourceId . '.json';

	$jsonDataCached = null; $cacheErrror = null;
	$jsonData = null; $apiError = null;
	
	// if we have cache file
	if (file_exists($tb_cache_file)) {

		// if cache is fresh enough
		if (time() - filemtime($tb_cache_file) <= $tb_cache_ttl_seconds) {

			// if we can read data from it
			if(($data = @file_get_contents($tb_cache_file)) != false) {
				$jsonDataCached = $data;
			}
			else {
				$cacheError = "Can't read data from cache file";
			}
		}
		else {
			$cacheError = "Cache file is older than $tb_cache_ttl_seconds seconds";
		}
	}
	else {
		$cacheError = "Cache file does not exist"; 
	}
	
	// if no cached data, try to get new data from Twitter using REST API
	if (!$jsonDataCached) {
	
		// if we have reached the limit, don't try to request data
		if (tb_rate_limit_remaining() <= 0) {
			$apiError = "Reached request limit";
		}
		else {
			
			// if it's a hashtag, use search API
			if (stripos($sourceId,'#') === 0) {
				$apiUrl = 'http://search.twitter.com/search.json?q=' . urlencode($sourceId);
			}
			// else assume it's a screen name
			else {
				$apiUrl = 'http://twitter.com/statuses/user_timeline.json?screen_name=' . $sourceId;
			}
			
			// try to get data from Twitter
			if ($data = @file_get_contents($apiUrl)) {
				$jsonData = $data;
			}
			// if failed, get the error
			else {
		        // Retrieve HTTP status code
		        list($version,$status_code,$msg) = explode(' ',$http_response_header[0], 3);
		
		        // Check the HTTP Status code
		        switch($status_code) {
		            case 400:
		                    $apiError = "Invalid request (400). You may have exceeded your rate limit.";
		                    break;
		            case 404:
		                    $apiError = "Not found (404).";
		                    break;
		            case 500:
		                    $apiError = "Server error (500). Twitter servers replied with an error. Hopefully they will be OK soon!";
		                    break;
		            case 503:
		                    $apiError = "Service unavilable (503). Twitter service is not available. Hopefully it will be OK soon!";
		                    break;
		            default:
		                    $apiError = "HTTP error " . $status_code;
		                    break;
		        }
			}
		}
	}
	
	// if we have fresh data, update cache and return it
	if ($jsonData) {
		if ($cf = fopen($tb_cache_file,'w')) {
			fwrite($cf,$jsonData);
			fclose($cf);
		}
		
		$json = new Services_JSON();
		return $json->decode($jsonData);		
	}
	// if we have cached data, return it
	else if($jsonDataCached) {
		$json = new Services_JSON();
		return $json->decode($jsonDataCached);		
	}
	// else report error
	else {
		return 'REST API: ' . $apiError . ' Cache: ' . $cacheError;
	}
}

function tb_rate_limit_remaining() {
	// if we could get it, return the actual number
	if (($jsonData = @file_get_contents('http://twitter.com/account/rate_limit_status.json')) != false) {
		$json = new Services_JSON();
		$limitResponseData = $json->decode($jsonData);
		return $limitResponseData->{'remaining_hits'};
	}
	// else, assume nothing is left 
	else {
		return 0;
	}
}

// search: Wed, 27 May 2009 15:52:40 +0000
// user feed: Thu May 21 00:09:16 +0000 2009
function tb_strtotime($date_string) {
	$mnum = array(
		'Jan' => 1,'Feb' => 2,'Mar' => 3,'Apr' => 4,'May' => 5,'Jun' => 6,
		'Jul' => 7,'Aug' => 8,'Sep' => 9 ,'Oct' => 10,'Nov'=>11,'Dec'=>12);
		
	if (strpos($date_string, ',') !== false) {
		list($wday,$mday, $mon, $year, $hour,$min,$sec,$offset) = preg_split('/[\s\:]/',$date_string);
	}
	else {
		list($wday,$mon,$mday,$hour,$min,$sec,$offset,$year) = preg_split('/[\s\:]/',$date_string);
	}
	
	return gmmktime($hour,$min,$sec,$mnum[$mon],$mday,$year);
}

function tb_verbal_time($timestamp) {
    $periods = array("second", "minute", "hour", "day", "week", "month", "year", "decade");
    $lengths = array("60","60","24","7","4.35","12","10");
   
    $now = time();
	$difference = $now - $timestamp;
   
    for($j = 0; $difference >= $lengths[$j] && $j < count($lengths)-1; $j++) {
        $difference /= $lengths[$j];
    }
    $difference = round($difference);
   
    if($difference != 1) {
        $periods[$j] .= "s";
    }
   
    return "$difference $periods[$j] ago";
}

include_once('admin-page.php');
?>
